WITH
--------------------apply regex operations to transactions-----------------------------------------------------------------------------------
rx AS (
SELECT 
    t.srce,
    t.id,
    t.rec,
    m.target,
    m.seq,
    regex->'regex'->>'function' regex_function,
    defn.v ->> 'field'          result_key_name,
    defn.v ->> 'key'            target_json_path,
    defn.v ->> 'flag'           regex_options_flag,
    defn.v->>'map'              map_intention,
    defn.v->>'retain'           retain_result,
    defn.v->>'regex'            regex_expression,
    defn.rn                     target_item_number,
    COALESCE(mt.rn,rp.rn,1)     result_number,
    mt.mt                       rx_match,
    rp.rp                       rx_replace,
    CASE defn.v->>'map'
        WHEN 'y' THEN
            defn.v->>'field'
        ELSE
            null
    END                         map_key,
    CASE defn.v->>'map'
        WHEN 'y' THEN
            CASE regex->'regex'->>'function'
                WHEN 'extract' THEN
                    CASE WHEN array_upper(mt.mt,1)=1 
                        THEN to_json(mt.mt[1])
                        ELSE array_to_json(mt.mt)
                    END::jsonb
                WHEN 'replace' THEN
                    to_jsonb(rp.rp)
                ELSE
                    '{}'::jsonb
            END
        ELSE
            NULL
    END                         map_val,
    CASE defn.v->>'retain'
        WHEN 'y' THEN
            defn.v->>'field'
        ELSE
            NULL
    END                         retain_key,
    CASE defn.v->>'retain'
        WHEN 'y' THEN
            CASE regex->'regex'->>'function'
                WHEN 'extract' THEN
                    CASE WHEN array_upper(mt.mt,1)=1 
                        THEN to_json(trim(mt.mt[1]))
                        ELSE array_to_json(mt.mt)
                    END::jsonb
                WHEN 'replace' THEN
                    to_jsonb(rtrim(rp.rp))
                ELSE
                    '{}'::jsonb
            END
        ELSE
            NULL
    END                         retain_val
FROM 
    --------------------------start with all regex maps------------------------------------------------------------------------------------
    tps.map_rm m
    --------------------------isolate matching basis to limit map to only look at certain json---------------------------------------------
    LEFT JOIN LATERAL jsonb_array_elements(m.regex->'regex'->'where') wh(v) ON TRUE
    --------------------------join to main transaction table but only certain key/values are included--------------------------------------
    INNER JOIN tps.trans t ON 
        t.srce = m.srce AND
        t.rec @> wh.v
    --------------------------break out array of regluar expressions in the map------------------------------------------------------------
    LEFT JOIN LATERAL jsonb_array_elements(m.regex->'regex'->'defn') WITH ORDINALITY defn(v, rn) ON true
    --------------------------each regex references a path to the target value, extract the target from the reference and do regex---------
    LEFT JOIN LATERAL regexp_matches(t.rec #>> ((defn.v ->> 'key')::text[]), defn.v ->> 'regex'::text,COALESCE(defn.v ->> 'flag','')) WITH ORDINALITY mt(mt, rn) ON
        m.regex->'regex'->>'function' = 'extract'
    --------------------------same as above but for a replacement type function------------------------------------------------------------
    LEFT JOIN LATERAL regexp_replace(t.rec #>> ((defn.v ->> 'key')::text[]), defn.v ->> 'regex'::text, defn.v ->> 'replace'::text,defn.v ->> 'flag') WITH ORDINALITY rp(rp, rn) ON
        m.regex->'regex'->>'function' = 'replace'
WHERE
    --t.allj IS NULL
    t.srce = 'dcard'
    --rec @> '{"Transaction":"ACH Credits","Transaction":"ACH Debits"}'
    --rec @> '{"Description":"CHECK 93013270 086129935"}'::jsonb
ORDER BY 
    t.id DESC,
    m.target,
    defn.rn,
    COALESCE(mt.rn,rp.rn,1)
)
--SELECT * FROM rx
--SELECT count(*) FROM rx LIMIT 100
, agg_to_target_items AS (
SELECT 
    srce
    ,id
    ,target
    ,seq
    ,map_intention
    ,regex_function
    ,target_item_number
    ,result_key_name
    ,target_json_path
    ,CASE WHEN map_key IS NULL 
        THEN    
            NULL 
        ELSE 
            jsonb_build_object(
                map_key,
                CASE WHEN max(result_number) = 1
                    THEN
                        jsonb_agg(map_val ORDER BY result_number) -> 0
                    ELSE
                        jsonb_agg(map_val ORDER BY result_number)
                END
            ) 
    END map_val
    ,CASE WHEN retain_key IS NULL 
        THEN 
            NULL 
        ELSE 
            jsonb_build_object(
                retain_key,
                CASE WHEN max(result_number) = 1
                    THEN
                        jsonb_agg(retain_val ORDER BY result_number) -> 0
                    ELSE
                        jsonb_agg(retain_val ORDER BY result_number)
                END
            ) 
    END retain_val
FROM 
    rx
GROUP BY
    srce
    ,id
    ,target
    ,seq
    ,map_intention
    ,regex_function
    ,target_item_number
    ,result_key_name
    ,target_json_path
    ,map_key
    ,retain_key
)
, agg_to_target AS (
SELECT
    srce
    ,id
    ,target
    ,seq
    ,map_intention
    ,tps.jsonb_concat_obj(COALESCE(map_val,'{}'::JSONB)) map_val
    ,jsonb_strip_nulls(tps.jsonb_concat_obj(COALESCE(retain_val,'{}'::JSONB))) retain_val
FROM
    agg_to_target_items
GROUP BY
    srce
    ,id
    ,target
    ,seq
    ,map_intention
ORDER BY
    id
)
, link_map AS (
SELECT
    a.srce
    ,a.id
    ,a.target
    ,a.seq
    ,a.map_intention
    ,a.map_val
    ,a.retain_val retain_value
    ,v.map
FROM
    agg_to_target a
    LEFT OUTER JOIN tps.map_rv v ON
        v.srce = a.srce AND
        v.target = a.target AND
        v.retval = a.map_val
)
--SELECT * FROM rx
--SELECT * FROM agg_to_target_items
--SELECT * FROM agg_to_target
SELECT * FROM link_map
